# Copyright 2008 Canonical Ltd.  All rights reserved.

"""Tests of lazr.restful navigation."""

__metaclass__ = type

from pkg_resources import resource_filename
import os
import unittest

from van.testing.layer import zcml_layer
from zope.component import getMultiAdapter
from zope.interface import Interface

from lazr.restful._resource import ReadWriteResource, UnknownEntryAdapter
from lazr.restful.error import expose, ClientErrorView, SystemErrorView
from lazr.restful.interfaces import IClientErrorView
from lazr.restful.testing.webservice import FakeRequest


class TestResource(ReadWriteResource):

    def __init__(self, callable, request):
        self.callable = callable
        super(TestResource, self).__init__(context=None, request=request)

    def do_GET(self):
        return self.callable()


class ErrorsTestCase(unittest.TestCase):

    def test_client_error_view_lookup(self):
        # An error decorated with lazr.restful.error.exposed will be adaptable
        # to IClientErrorView.
        request = FakeRequest(version='trunk')
        exception_view = getMultiAdapter(
            (expose(ValueError()), request), IClientErrorView)
        self.assertEquals(exception_view.__class__, ClientErrorView)

    def test_undecorated_exception(self):
        # If an undecorated exception (a regular Python exception) is
        # generated during a request the exception percolates all the way out
        # of the resource (to be handled by the publisher).

        def broken():
            raise RuntimeError('something broke')

        request = FakeRequest(version='trunk')
        resource = TestResource(broken, request)
        self.assertRaises(RuntimeError, resource)

    def test_decorated_exception(self):
        # If an undecorated exception is generated during a request the
        # SystemErrorView is used to generate the response.

        def broken():
            raise expose(RuntimeError('something broke'))

        request = FakeRequest(version='trunk')
        resource = TestResource(broken, request)
        result = resource()
        # The client error view sets the status code to 400 (client error).
        self.assertEqual(request.response.getStatus(), 400)
        # The message used to construct the exception is included in the body
        # of the response.
        self.assert_(result.startswith('something broke'))

    def test_passing_bad_things_to_expose(self):
        # The expose function only accepts instances of exceptions.  It
        # generates errors otherwise.
        self.assertRaises(ValueError, expose, 1)
        self.assertRaises(ValueError, expose, 'x')
        self.assertRaises(ValueError, expose, RuntimeError)

    def test_missing_adapter(self):
        # If there is no multi-adapter from the entry interface (IMyEntry) and
        # a request to IEntry an exception is raised.
        class IMyEntry(Interface):
            pass

        # Since the test wants to inspect the exception message (below) we're
        # not using self.assertRaises).
        try:
            EntryAdapterUtility.forSchemaInterface(
                IMyEntry, self.beta_request)
        except Exception, e:
            pass
        self.assert_(isinstance(e, Exception))

        # The exception's message explains what went wrong.
        self.assertTrue(str(e),
            'No IEntry adapter found for IMyEntry (web service version: beta).')

    def test_missing_adapter_whence(self):
        # The UnknownEntryAdapter exception has a "whence" attribute that
        # higher-level code can set to give the reader of the exeption message
        # a hint about where in the code the mistake was made that triggered
        # the exception.
        exception = UnknownEntryAdapter('IInterfaceInQuestion', '2.0')
        exception.whence = 'Encounterd as a result of badness in foo.py.'
        self.assertEquals(str(exception),
            'No IEntry adapter found for IInterfaceInQuestion (web service '
            'version: 2.0).  Encounterd as a result of badness in foo.py.')


class FunctionalLayer:
    allow_teardown = False
    zcml = os.path.abspath(resource_filename('lazr.restful', 'ftesting.zcml'))


zcml_layer(FunctionalLayer)


def additional_tests():
    """See `zope.testing.testrunner`."""
    suite = unittest.TestLoader().loadTestsFromName(__name__)
    suite.layer = FunctionalLayer
    return suite
